//----------------------------------------------------------------------------
/*
 * File: apputils.c
 * - some useful functions
 * 
 * Part of Emulation of PMD 32 drive.
 * Based on MMC/SD/SDHC card library by Roland Riegel
 *   - http://www.roland-riegel.de/sd-reader/
 *
 * Author: Roman Brik { pmd85emu (at) gmail.com }
 * Web: http://pmd85.borik.net
 * Date: 10 Oct 2009
 * Version 2.0
*/
//----------------------------------------------------------------------------
#include <string.h>

#include "apputil.h"
//-----------------------------------------------------------------------------
char* apputil_trim_left(char *in_out)
{
  char *p, *q;

  if (*in_out == ' ')
  {
    p = in_out;
    while (*p == ' ')
      p++;

    q = in_out;
    while (*p)
      *q++ = *p++;
    *q = 0;
  }

  return in_out;
}
//-----------------------------------------------------------------------------
char* apputil_trim_right(char *in_out)
{
  char *p;

  p = &in_out[strlen(in_out)];
  while (p > in_out)
  {
    p--;
    if (*p == ' ')
      *p = 0;
    else
      break;
  }

  return in_out;
}
//-----------------------------------------------------------------------------
char* apputil_trim(char *in_out)
{
  return apputil_trim_left(apputil_trim_right(in_out));
}
//-----------------------------------------------------------------------------
char* apputil_left_text(char *out, const char *in, uint8_t width)
{
  uint8_t w;

  memset(out, ' ', width);
  out[width] = 0;

  w = strlen(in);
  if (w > width)
    w = width;
  strncpy(out, in, w);

  return out;
}
//-----------------------------------------------------------------------------
char* apputil_right_text(char *out, const char *in, uint8_t width)
{
  uint8_t w;

  memset(out, ' ', width);
  out[width] = 0;

  w = strlen(in);
  if (w > width)
    w = width;
  strncpy(out + (width - w), in, w);

  return out;
}
//-----------------------------------------------------------------------------
char* apputil_center_text(char *out, const char *in, uint8_t width)
{
  uint8_t w;

  memset(out, ' ', width);
  out[width] = 0;

  w = strlen(in);
  if (w > width)
    w = width;
  strncpy(out + (width - w) / 2, in, w);

  return out;
}
//-----------------------------------------------------------------------------
char* apputil_left_text_p(char *out, PGM_P in, uint8_t width)
{
  uint8_t w;

  memset(out, ' ', width);
  out[width] = 0;

  w = strlen_P(in);
  if (w > width)
    w = width;
  strncpy_P(out, in, w);

  return out;
}
//-----------------------------------------------------------------------------
char* apputil_right_text_p(char *out, PGM_P in, uint8_t width)
{
  uint8_t w;

  memset(out, ' ', width);
  out[width] = 0;

  w = strlen_P(in);
  if (w > width)
    w = width;
  strncpy_P(out + (width - w), in, w);

  return out;
}
//-----------------------------------------------------------------------------
char* apputil_center_text_p(char *out, PGM_P in, uint8_t width)
{
  uint8_t w;

  memset(out, ' ', width);
  out[width] = 0;

  w = strlen_P(in);
  if (w > width)
    w = width;
  strncpy_P(out + (width - w) / 2, in, w);

  return out;
}
//----------------------------------------------------------------------------
char* apputil_to_dec(char *out, uint32_t val, char fill, uint8_t minwidth)
{
  char *p = out;
  uint32_t num = 1000000000;
  uint32_t minnum = 1;
  uint8_t valid = 0;
  uint8_t b;

  if (minwidth < 1 || minwidth > 10)
  {
    *out = 0;
    return out;
  }

  while (--minwidth > 0)
    minnum *= 10;

  while (num > 0)
  {
    b = val / num;
    if (b > 0 || valid || num <= minnum)
    {
      if (b == 0 && !valid)
      {
        if (fill)
          *p++ = fill;
        else if (num <= minnum)
          *p++ = '0';
      }
      else
      {
        *p++ = ('0' + b);
        valid = 1;
      }
    }

    val -= (b * num);
    num /= 10;
  }
  *p = 0;

  return out;
}
//----------------------------------------------------------------------------
char* apputil_to_hex(char *out, uint32_t val, char fill, uint8_t minwidth)
{
  char *p = out;
  uint32_t num = 0x10000000;
  uint32_t minnum = 1;
  uint8_t valid = 0;
  uint8_t b;

  if (minwidth < 1 || minwidth > 8)
  {
    *out = 0;
    return out;
  }

  while (--minwidth > 0)
    minnum *= 16;

  while (num > 0)
  {
    b = val / num;
    if (b > 0 || valid || num <= minnum)
    {
      if (b == 0 && !valid)
      {
        if (fill)
          *p++ = fill;
        else if (num <= minnum)
          *p++ = '0';
      }
      else
      {
        if (b < 10)
          *p++ = ('0' + b);
        else
          *p++ = ('A' + b - 10);
        valid = 1;
      }
    }

    val -= (b * num);
    num /= 16;
  }
  *p = 0;

  return out;
}
//----------------------------------------------------------------------------
// out - output buffer to store extracted name without extention
// max - length of output buffer
// in - input file path/name
char* apputil_extract_file_name(char *out, uint8_t max, const char *in)
{
  char *ps, *pd;

  memset(out, 0, max);
  ps = strrchr(in, '/');
  if (ps == 0)
    ps = (char *)in;
  else
    ps++;
  pd = strchr(ps, '.');
  if (pd == 0 || ((pd - ps) >= max))
    memcpy(out, ps, max - 1);
  else
    memcpy(out, ps, pd - ps);

  return out;
}
//----------------------------------------------------------------------------
// file_name - name of file to check extention
// ext - pointer to extention without dot in pgm
uint8_t apputil_check_file_ext_p(char *file_name, PGM_P ext)
{
  char *pd;

  pd = strrchr(file_name, '.');
  if (pd == 0)
    return 0;

  return (strcasecmp_P(pd + 1, ext) == 0);
}
//----------------------------------------------------------------------------
// file_name - name of file to check extention
// ext - pointer to extention without dot in RAM
uint8_t apputil_check_file_ext(char *file_name, char *ext)
{
  char *pd;

  pd = strrchr(file_name, '.');
  if (pd == 0)
    return 0;

  return (strcasecmp(pd + 1, ext) == 0);
}
//----------------------------------------------------------------------------
