//----------------------------------------------------------------------------
/*
 * File: lcd.h
 * - header file for lcd.c
 * 
 * Part of Emulation of PMD 32 drive.
 * Based on MMC/SD/SDHC card library by Roland Riegel
 *   - http://www.roland-riegel.de/sd-reader/
 *
 * Author: Roman Brik { pmd85emu (at) gmail.com }
 * Web: http://pmd85.borik.net
 * Date: 10 Oct 2009
 * Version 2.0
*/
//----------------------------------------------------------------------------
#ifndef __LCD_H__
#define __LCD_H__
//----------------------------------------------------------------------------
#define LCD_PORT_CTRL           DDRD
#define LCD_PORT_DATA           PORTD

#define LCD_DATA_MASK           0x3C
#define lcd_data_config()       DDRD |= LCD_DATA_MASK

#define LCD_CMD                 0
#define LCD_DATA                1
#define lcd_rs_config()         DDRD |= (1 << DDD6)
#define lcd_rs_cmd()            PORTD &= ~(1 << PD6)
#define lcd_rs_data()           PORTD |= (1 << PD6)

#define lcd_e_config()          DDRB |= (1 << DDB3)
#define lcd_e_on()              PORTB |= (1 << PB3)
#define lcd_e_off()             PORTB &= ~(1 << PB3)

#define lcd_backlight_config()  DDRB |= (1 << DDB2)
#define lcd_backlight_on()      PORTB &= ~(1 << PB2)
#define lcd_backlight_off()     PORTB |= (1 << PB2)
#define lcd_is_backlight_on()   (PORTB & (1 << PB2)) ? 0 : 1

#define LCD_CMD_CLR             0x01  // clear display
#define LCD_CMD_HOME            0x02  // return to home position
#define LCD_CMD_ENTRY_MODE      0x06  // set entry mode - increment, no shift
#define LCD_CMD_DISPLAY_OFF     0x08  // display OFF
#define LCD_CMD_DISPLAY_ON      0x0C  // display ON, cursor OFF
#define LCD_CMD_CURSOR_ON       0x0E  // display ON, cursor ON
#define LCD_CMD_CURSOR_BLINK    0x0F  // display ON, cursor blink
#define LCD_CMD_CURSOR_LEFT     0x10  // shift cursor left 
#define LCD_CMD_CURSOR_RIGHT    0x14  // shift cursor right
#define LCD_CMD_SHIFT_LEFT      0x18  // shift display left 
#define LCD_CMD_SHIFT_RIGHT     0x1C  // shift display right
#define LCD_CMD_FUNCTION        0x28  // function set - 4 bit, 2 lines
#define LCD_CMD_CGRAM_ADDR      0x40  // set CGRAM address
#define LCD_CMD_DDRAM_ADDR      0x80  // set DDRAM address

// cursor position to DDRAM mapping
#define LCD_LINE0_DDRAM_ADDR    0x00
#define LCD_LINE1_DDRAM_ADDR    0x40
#define LCD_LINE2_DDRAM_ADDR    0x14
#define LCD_LINE3_DDRAM_ADDR    0x54

#define LCD_DISPLAY_ROWS        2
#define LCD_DISPLAY_COLUMNS     16
//----------------------------------------------------------------------------
void lcd_init(void);
void lcd_on(void);
void lcd_off(void);
void lcd_clr(void);
void lcd_cursor_on(void);
void lcd_cursor_off(void);
void lcd_cursor_blink(void);
void lcd_home(void);
void lcd_set_position(uint8_t row, uint8_t col);
void lcd_shift_left(uint8_t n);
void lcd_shift_right(uint8_t n);
void lcd_cursor_left(uint8_t n);
void lcd_cursor_right(uint8_t n);
void lcd_define_char(const prog_uint8_t *pc, uint8_t char_code);
void lcd_disp_char(char c);
void lcd_disp_char_at(char c, uint8_t row, uint8_t col);
void lcd_disp_string(char *str);
void lcd_disp_string_at(char *str, uint8_t row, uint8_t col);
void lcd_disp_string_p(PGM_P str);
void lcd_disp_string_at_p(PGM_P str, uint8_t row, uint8_t col);
//----------------------------------------------------------------------------
#endif
//----------------------------------------------------------------------------
