//----------------------------------------------------------------------------
/*
 * File: pmd32.h
 * - header file for pmd32.c
 *
 * Part of Emulation of PMD 32 drive.
 * Based on MMC/SD/SDHC card library by Roland Riegel
 *   - http://www.roland-riegel.de/sd-reader/
 *
 * Author: Roman Brik { pmd85emu (at) gmail.com }
 * Web: http://pmd85.borik.net
 * Date: 10 Oct 2009
 * Version 2.0
*/
//----------------------------------------------------------------------------
#ifndef __PMD32_H__
#define __PMD32_H__
//----------------------------------------------------------------------------
#define PMD32_DATA_PORT_DIR     DDRC
#define PMD32_DATA_PORT_DATA    PORTC
#define PMD32_DATA_PORT_PIN     PINC

#define PMD32_CTRL_PORT_DIR     DDRA
#define PMD32_CTRL_PORT_DATA    PORTA
#define PMD32_CTRL_PORT_PIN     PINA

#define PMD32_OBF               _BV(4)    // input
#define PMD32_IBF               _BV(5)    // input
#define PMD32_ACK               _BV(6)    // output
#define PMD32_STB               _BV(7)    // output
#define PMD32_DIR               _BV(7)    // output

#define pmd32_ctrl_config()     PMD32_CTRL_PORT_DATA |= (PMD32_OBF | PMD32_IBF | PMD32_ACK | PMD32_STB); \
                                PMD32_CTRL_PORT_DIR = (PMD32_CTRL_PORT_DIR & (~(PMD32_OBF | PMD32_IBF))) | (PMD32_ACK | PMD32_STB)

#define pmd32_dir_config()      PORTD |= PMD32_DIR; DDRD |= PMD32_DIR

#define pmd32_data_port_Z()     PMD32_DATA_PORT_DIR = 0; PMD32_DATA_PORT_DATA = 0xFF
#define pmd32_data_port_out()   PMD32_DATA_PORT_DIR = 0xFF
#define pmd32_data_write(val)   PMD32_DATA_PORT_DATA = val
#define pmd32_data_read()       PMD32_DATA_PORT_PIN

#define pmd32_dir_output()      PORTD &= ~PMD32_DIR
#define pmd32_dir_input()       PORTD |= PMD32_DIR

#define pmd32_ack_on()          PMD32_CTRL_PORT_DATA &= ~PMD32_ACK
#define pmd32_ack_off()         PMD32_CTRL_PORT_DATA |= PMD32_ACK

#define pmd32_stb_on()          PMD32_CTRL_PORT_DATA &= ~PMD32_STB
#define pmd32_stb_off()         PMD32_CTRL_PORT_DATA |= PMD32_STB

#define pmd32_byte_accepted()   !(PMD32_CTRL_PORT_PIN & PMD32_IBF)
#define pmd32_byte_available()  !(PMD32_CTRL_PORT_PIN & PMD32_OBF)


#define SEND_TIMEOUT            30
#define SEND_PRESENT_TIMEOUT    15
#define SEND_RESULT_TIMEOUT     50
#define SEND_ACK_TIMEOUT        500
#define SEND_NAK_TIMEOUT        0

#define WAIT_PRESENT_TIMEOUT    10
#define WAIT_COMMAND_TIMEOUT    30
#define WAIT_BYTE_TIMEOUT       5

#define PRESENTATION            0xAA
#define ACK                     0x33
#define NAK                     0x99

#define OP_NO_TRANSFER          0
#define OP_SEND_DATA            1
#define OP_RECEIVE_DATA         2
#define OP_FORMAT               3
#define OP_ERROR                -1

#define RESULT_OK               0         // OK
#define RESULT_WP               1         // disk is write protected
#define RESULT_FE               2         // format error
#define RESULT_RE               3         // read error
#define RESULT_WE               4         // write error
#define RESULT_BD               5         // bad drive number
#define RESULT_NF               6         // file/dir not found
#define RESULT_TL               7         // file/dir name too long
#define RESULT_CE               8         // create image error
#define RESULT_CM               9         // config file missing
#define RESULT_CR               10        // config file read error
#define RESULT_CC               11        // config file corrupted
#define RESULT_UI               12        // unknown image type

#define SECTOR_SIZE             128
#define SECTORS_PER_PHYSICS     4
#define PHYSICAL_SECTOR_SIZE    (SECTORS_PER_PHYSICS * SECTOR_SIZE)

#define DRIVES_COUNT            4
#define DRIVE_A                 0
#define DRIVE_B                 1
#define DRIVE_C                 2
#define DRIVE_D                 3

#define CFG_MAX_LINE_LENGTH     32
#define NUL                     0
#define TAB                     9
#define LF                      10
#define CR                      13
#define EOF                     26
#define SPC                     32
//----------------------------------------------------------------------------
void pmd32_main(struct fat_fs_struct *fs);
//----------------------------------------------------------------------------
#endif
//----------------------------------------------------------------------------
